#!/bin/sh
# How many modules should we compile?
MODULE_COUNT=100
# How many statically initialized variables should we put in each TU?
STATIC_COUNT=200
# How many dynamically initialized variables should we put in each TU?
DYNAMIC_COUNT=200

# Where is the clang executable located?
CXX=$1

if [ $# -eq 0 ]
then
  echo "Usage: $0 [path to clang++ built with initialization order checking]"
  exit
fi

if [ $# -eq 4 ]
then
  MODULE_COUNT=$2
  STATIC_COUNT=$3
  DYNAMIC_COUNT=$4
fi

CANONICAL_CXX=$(readlink -f $CXX)
echo "Using $CANONICAL_CXX"

#Print some info about what we're doing
echo -n "Generating $MODULE_COUNT modules with $STATIC_COUNT statically "
echo -n "initialized variables each and $DYNAMIC_COUNT dynamically initialized "
echo    "variables each."
TOTAL_STATIC=$(($MODULE_COUNT * $STATIC_COUNT))
TOTAL_DYNAMIC=$(($MODULE_COUNT * $DYNAMIC_COUNT))
echo "TOTAL: $TOTAL_STATIC static variables, $TOTAL_DYNAMIC dynamic variables" 
echo

# Generate our files using a quick C++ script
BUILD_DIR=$(mktemp -d)
GENERATOR=$BUILD_DIR/stress_initialiers
$CXX -o $GENERATOR stress_initializers.cpp
(cd $BUILD_DIR && exec $GENERATOR $MODULE_COUNT $STATIC_COUNT $DYNAMIC_COUNT)

# Test how much we're affecting both compilation and runtime
ASAN_FLAG="-faddress-sanitizer"
INIT_FLAG="-mllvm -asan-initialization-order"
for CFLAGS in "" "$ASAN_FLAG" "$ASAN_FLAG $INIT_FLAG"
do
  echo -n "Compiling with CFLAGS: [ $CFLAGS ]"
  time $CXX $CFLAGS $BUILD_DIR/*.cc -o $BUILD_DIR/output
  echo -n "Compile complete, testing runtime"
  time ASAN_OPTIONS=initialization_order=1 $BUILD_DIR/output
done

rm -r $BUILD_DIR
